/************************************************************************
 * Steve Wendt
 * CSCI 231 - Project 1
 * This program does the following:
 *      opens a game window labelled "OpenGL Pong"
 *      has context menu for right mouse button
 *      draws walls and paddles according to number of players
 *      moves ball around according to collision with walls/paddles
 ************************************************************************/

#include <GL/glut.h>
#include <stdlib.h>    //NULL
#include <string.h>    //strlen()

const int MAXPLAYERS=4;        //maximum of 4 players
int       numplayers=1;        //one player by default

struct Players{
    int score;
    float r, g, b, x1, x2, y1, y2;
    inline void SetInfo(float red, float green, float blue, 
                        float xcoord1, float ycoord1, float xcoord2, float ycoord2){
        r=red, g=green, b=blue, x1=xcoord1, y1=ycoord1, x2=xcoord2, y2=ycoord2;
    }
}players[MAXPLAYERS];

float ballx=250, bally=250, ballz=0;       //ball coordinates

void drawString(float xpos, float ypos, char string[]){
    for(unsigned int i=0; i<strlen(string); i++){
        glRasterPos2f(xpos, ypos);
        glutBitmapCharacter(GLUT_BITMAP_9_BY_15, string[i]);
        //xpos += glutBitmapWidth(GLUT_BITMAP_9_BY_15, string[i]);
        xpos += 10;
    }
}

void display(){
    glClear(GL_COLOR_BUFFER_BIT);

    //draw the title
    glColor3f(1.0, 1.0, 1.0);
    drawString(200.0, 585.0, "OpenGL Pong\0");

    //draw the walls
    glColor3f(0.0, 0.0, 1.0);
    if(numplayers < 1){
        glRectf(0.0, 0.0, 10.0, 500.0);
        glRectf(0.0, 0.0, 500.0, 10.0);
        glRectf(0.0, 490.0, 500.0, 500.0);
        glRectf(490.0, 0.0, 500.0, 500.0);
    }
    else if(numplayers < 2){
        glRectf(25.0, 0.0, 500.0, 10.0);
        glRectf(25.0, 490.0, 500.0, 500.0);
        glRectf(490.0, 0.0, 500.0, 500.0);
    }
    else if(numplayers < 3){
        glRectf(25.0, 0.0, 475.0, 10.0);
        glRectf(25.0, 490.0, 475.0, 500.0);
    }
    else if(numplayers < 4)
        glRectf(25.0, 490.0, 475.0, 500.0);

    //draw the ball
    glPushMatrix();
    glColor3f(1.0, 0.0, 0.0);
    glTranslatef(ballx, bally, ballz);
    glutSolidSphere(6, 25, 25);
    glPopMatrix();

    //draw the paddles
    for(int i=0; i<numplayers; i++){
        glColor3f(players[i].r, players[i].g, players[i].b);
        glRectf(players[i].x1, players[i].y1, players[i].x2, players[i].y2);
    }

    //draw the score info
    glPushMatrix();
    glColor3f(0.0, 0.0, 0.5);
    glTranslatef(0, 0, -25);
    glRectf(0.0, 500.0, 500.0, 565.0);
    glPopMatrix();

    glColor3f(1.0, 0.5, 0.5);
    drawString(10.0, 545.0, "Scores\0");

    char score[8], buff[8];
    if(numplayers > 0){
        glColor3f(players[0].r, players[0].g, players[0].b);
        glRectf(75.0, 525.0, 100.0, 535.0);
        strcpy(score, ":\0");
        strcat(score, gcvt(players[0].score, 2, buff));
        drawString(105.0, 525.0, score);
    }
    if(numplayers > 1){
        glColor3f(players[1].r, players[1].g, players[1].b);
        glRectf(175.0, 525.0, 200.0, 535.0);
        strcpy(score, ":\0");
        strcat(score, gcvt(players[1].score, 2, buff));
        drawString(205.0, 525.0, score);
    }
    if(numplayers > 2){
        glColor3f(players[2].r, players[2].g, players[2].b);
        glRectf(275.0, 525.0, 300.0, 535.0);
        strcpy(score, ":\0");
        strcat(score, gcvt(players[2].score, 2, buff));
        drawString(305.0, 525.0, score);
    }
    if(numplayers > 3){
        glColor3f(players[3].r, players[3].g, players[3].b);
        glRectf(375.0, 525.0, 400.0, 535.0);
        strcpy(score, ":\0");
        strcat(score, gcvt(players[3].score, 2, buff));
        drawString(405.0, 525.0, score);
    }

    glutSwapBuffers();
}

void animate(){
    const  int width = 5;           //width of ball to account for
    static int incx = 5, incy = 0;  //incremental steps to move the ball

    int incrand = rand()%10 + 1;    //random incremental value from -5 to 5
    if(incrand > 5) incrand = -(incrand-5);

    //test for collisions
    if(numplayers>=1 && ballx<=15+width &&        //collision with player 1
        bally >= players[0].y1 && bally <= players[0].y2)
        incx = 5, incy = incrand, players[0].score++;

    else if(numplayers<1 && ballx <= 10+width)    //collision with wall 1
        incx = 5, incy = incrand;

    else if(ballx <= 10+width)                    //player 1 loses points
        players[0].score -= 5, incx = 5, incy = incrand, ballx = 250, bally = 250;

    else if(numplayers>=2 && ballx>=485-width &&  //collision with player 2
        bally >= players[1].y1 && bally <= players[1].y2)
        incx = -5, incy = incrand, players[1].score++;

    else if(numplayers<2 && ballx >= 490-width)   //collision with wall 2
        incx = -5, incy = incrand;

    else if(ballx >= 490-width)                   //player 2 loses points
        incx = -5, incy = incrand, players[1].score -= 5, ballx = 250, bally = 250;

    else if(numplayers>=3 && bally<=15+width &&   //collision with player 3
        ballx >= players[2].x1 && ballx <= players[2].x2)
        incx = incrand, incy = 5, players[2].score++;

    else if(numplayers<3 && bally <= 10+width)    //collision with wall 3
        incx = incrand, incy = 5;

    else if(bally <= 10+width)                    //player 3 loses points
        incx = incrand, incy = 5, players[2].score -= 5, ballx = 250, bally = 250;

    else if(numplayers>=4 && bally>=485-width &&  //collision with player 4
        ballx >= players[3].x1 && ballx <= players[3].x2)
        incx = incrand, incy = -5, players[3].score++;

    else if(numplayers<4 && bally >= 490-width)   //collision with wall 4
        incx = incrand, incy = -5;

    else if(bally >= 490-width)                   //player 4 loses points
        incx = incrand, incy = -5, players[3].score -= 5, ballx = 250, bally = 250;

    ballx+=incx, bally+=incy;  //new coordinates for ball

    glutPostRedisplay();
}

void reshape(int w, int h){
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if(w <= h)
        glOrtho(0.0, 500.0, 0.0, 600.0*(GLfloat)h/(GLfloat)w, -50.0, 50.0);
    else
        glOrtho(0.0, 500.0*(GLfloat)w/(GLfloat)h, 0.0, 600.0, -50.0, 50.0);
    glMatrixMode(GL_MODELVIEW);
}

void specKey(int key, int x, int y){
    switch (key){
        //Player 1 has Up/Down arrow keys
        case GLUT_KEY_UP:    //up key pressed
            if(players[0].y2 < 490)
                players[0].y1 += 10, players[0].y2 += 10;
            break;
        case GLUT_KEY_DOWN:  //down key pressed
            if(players[0].y1 > 10)
                players[0].y1 -= 10, players[0].y2 -= 10;
            break;

        //Player 3 has Left/Right arrow keys
        case GLUT_KEY_LEFT:    //left key pressed
            if(players[2].x1 > 10)
                players[2].x1 -= 10, players[2].x2 -= 10;
            break;
        case GLUT_KEY_RIGHT:  //right key pressed
            if(players[2].x2 < 490)
                players[2].x1 += 10, players[2].x2 += 10;
            break;
    }
    glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y){
    switch (key){
        //Player 2 has w/s keys
        case 'w':
        case 'W':    //up key pressed
            if(players[1].y2 < 490)
                players[1].y1 += 10, players[1].y2 += 10;
            break;
        case 's':
        case 'S':  //down key pressed
            if(players[1].y1 > 10)
                players[1].y1 -= 10, players[1].y2 -= 10;
            break;

        //Player 4 has a/d keys
        case 'a':
        case 'A':    //left key pressed
            if(players[3].x1 > 10)
                players[3].x1 -= 10, players[3].x2 -= 10;
            break;
        case 'd':
        case 'D':  //right key pressed
            if(players[3].x2 < 490)
                players[3].x1 += 10, players[3].x2 += 10;
            break;

        case 'q':  //get me out!
        case 'Q':
        case 27 :  //Esc key
            exit(0);
            break;
    }
}

void mouse(int btn, int state, int x, int y){
    if(btn==GLUT_LEFT_BUTTON   && state==GLUT_DOWN) glutIdleFunc(animate);
    if(btn==GLUT_MIDDLE_BUTTON && state==GLUT_DOWN) glutIdleFunc(NULL);
}

void right_menu(int menuitem){
    switch(menuitem){
        case 1:
            glutIdleFunc(animate);
            break;
        case 2:
            glutIdleFunc(NULL);
            break;
        case 3:
        default:
            exit(0);
    }
}

void numplyrs_menu(int menuitem){
    numplayers=menuitem;
    glutPostRedisplay();
}

void init(){
    //set up some callbacks
    glutDisplayFunc(display);
    glutReshapeFunc(reshape);
    glutMouseFunc(mouse);
    glutKeyboardFunc(keyboard);
    glutSpecialFunc(specKey);

    //create context menus
    int submenu=glutCreateMenu(numplyrs_menu);
    glutAddMenuEntry("Zero",  0);
    glutAddMenuEntry("One",   1);
    glutAddMenuEntry("Two",   2);
    glutAddMenuEntry("Three", 3);
    glutAddMenuEntry("Four",  4);    

    glutCreateMenu(right_menu);
    glutAddSubMenu("# of Players", submenu);
    glutAddMenuEntry("Start ball", 1);
    glutAddMenuEntry("Stop  ball", 2);
    glutAddMenuEntry("Exit", 3);
    glutAttachMenu(GLUT_RIGHT_BUTTON);

    //seed the randomization function
    srand(submenu);

    //set up the paddle info
    players[0].SetInfo(0.0, 1.0, 0.0, 10, 225, 20, 275);  
    players[1].SetInfo(1.0, 1.0, 0.0, 480, 225, 490, 275);
    players[2].SetInfo(0.0, 1.0, 1.0, 225, 10, 275, 20);  
    players[3].SetInfo(0.5, 0.5, 0.5, 225, 480, 275, 490);  
}

int main(int argc, char* argv[]){
    //set up our window
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize(500, 600);
    glutInitWindowPosition(50, 50);
    glutCreateWindow("OpenGL Pong version 0.4");

    init();
    glutMainLoop();
    return 0;
}
